importPackage(java.lang);
importPackage(java.sql);
importPackage(java.util);
importPackage(java.io);
importPackage(Packages.javax.xml);
importPackage(Packages.javax.xml.parsers);
importPackage(Packages.javax.xml.transform);
importPackage(Packages.javax.xml.transform.dom);
importPackage(Packages.javax.xml.transform.stream);
importPackage(Packages.org.w3c.dom);
importPackage(Packages.com.invoqsystems.integration.CM.model);

var remedyURL = "http://localhost";
var remedyServerName = "REMEDYCM";

/**
 * Replaces the full msg error string with a condensed version which lists the problem.
 * This method will remove all unneeded information in the error message.
 * Return: stripped down error message
 */
function escapeMessage( msg )
{
  if ( msg == null )
  {
    return null;
  }
  msg = msg.replaceAll( "\n"," " );
  msg = msg.replaceAll( ".*java.lang.Exception: WS Fault: soapenv:Server.userException [|]+[ ]+ERROR \\([0-9]+\\): [;]*", "" );
  return msg;
}

/**
 * Add Remdey Work Info
 *
 * Parameters: a java.util.Map<String, String>
 * Return: a error message or null if successful
 */
function addWorkInfo( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var addInfoFields = params.get( "addInfoFields" );
  if ( user == null  || addInfoFields == null )
  {
    return "One of the required fields (cmUser, addInfoFields) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return escapeMessage( cmo.addWorkInfo( addInfoFields ) );
  }
}
   
/**
 * Add an Approver to a Change Request
 * @param fields
 * @return
 * @throws Exception
 */
function addApprover( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var addApproverFields = params.get( "addApproverFields" );
  if ( user == null  || addApproverFields == null )
  {
    return "One of the required fields (cmUser, addApproverFields) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return escapeMessage( cmo.addApprover( addApproverFields ) );
  }
}

/**
 * Creates an incident (Change Request)
 *
 * Parameters: a java.util.Map<String, String>
 * Return: a error message or null if successful
 */
function createIncident( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var createFields = params.get( "createFields" );
  if ( user == null  || createFields == null )
  {
    return "One of the required fields (cmUser, createFields) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = cmo.createIncident( createFields );
    return escapeMessage( msg );
  }
}

/**
 * Query against Remedy WS for a count of all incidents that passes the passed in Qualification
 *
 * Parameters: a java.util.Map<String, String> containing 
 *             "Qualification", The qualification used in the Remedy search
  * Return: A java.lang.Integer 
 */
function getIncidentCount( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null )
  {
    return "One of the required fields (cmUser, Qualification) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return new Integer( cmo.queryIncidentIDs( qual, null, null ).size() );
  }
}

/**
 * Retrieve the Incident Details for a specific Change Request
 * Parameters: a java.util.Map<String, String>
 *
 * Return: Change Request Details
 */
function getIncidentDetail( params )
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var incidentNumber = params.get( "IncidentNumber" );  
  if ( user == null  || incidentNumber == null )
  {
    return "One of the required fields (cmUser, IncidentNumber) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );  
    return cmo.queryIncidentDetail( incidentNumber );
  }
}

/**
 * Query against Remedy WS for header of all incidents that passes the passed in Qualification
 * A header consist of <Infrastructure_Change_ID, Impact, Urgency, Description>  
 *
 * Parameters: a java.util.Map<String, String> containing 
 *             "Qualification", The qualification used in the Remedy search
  * Return: Header information of the matched incidents.
 */
function getIncidentHeaders( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null )
  {
    return "One of the required fields (cmUser, Qualification) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    try
    {
      cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
      return cmo.queryIncidentHeaders( qual, null, null );
    }
    catch (ex)
    {
      var msg = ex.toString();
      if ( msg.indexOf( "Qualification line error" ) != -1 )
      {
        return null;
      }
      return msg;
    }
  }
}

/**
 * Provides a method to search the Resolution Categories 
 *
 * Parameters: a java.util.Map<String, String>
 *
 * Return: a error message or null if successful
 */
function queryCGM( params )
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null )
  {
    return "One of the required fields (cmUser, Qualification) has no value.";
  }
  else
  {    
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );

    var operation = "MG_GCM_QueryList";
    var service = "APINT_Generic_Module_WS";
    var valuesNode = "getListValues";
    try
    {
      return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
    }
    catch (e)
    {
      return e.toString();
    }      
  }
}

/**
 * Queries the generic list methods, this should be used when you are using qualifications to return a web service list operation.
 *
 * Parameters: a java.util.Map<String, String>
 * Return: A Generic List, List<GenericWSObject>
 */
function queryGenericList( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var serv = params.get( "service" );
  var qual = params.get( "Qualification" );
  if ( user == null  || qual == null || serv == null)
  {
    return "One of the required fields (cmUser, Qualification, service) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    try
    {
      cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
      if ( "Requester".equalsIgnoreCase( serv ) )
      {
        return cmo.queryPeople( qual, null, null );
      }
      else if ( "Group".equalsIgnoreCase( serv ) )
      {
        return cmo.queryGroup( qual, null, null );
      }
      else if ( "VendorGroup".equalsIgnoreCase( serv ) )
      {
        qual = "('Vendor Group'=\"Yes\") AND (" + qual + ")";
        return cmo.queryGroup( qual, null, null );
      }
      else if ( "ChangeManager".equalsIgnoreCase( serv ) )
      {
        return cmo.queryChangeManagerAssociation( qual, null, null );
      }
      else if ( "ChangeAssignee".equalsIgnoreCase( serv ) )
      {
        return cmo.queryChangeManagerAssociation( qual, null, null );
      }
      else if ( "Assignee".equalsIgnoreCase( serv ) )
      {
        return cmo.queryGroupAssociation( qual, null, null );
      }
      else if ( "Reason".equals( serv ) )
      {
        var operation = "SYS_StatusReason_QueryList";
        var service = "APINT_StatusReason_WS";
        var valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "Company".equals( serv ) )
      {
        var operation = "APINT_AllCom_QueryList";
        var service = "APINT_COM_Company_WS";
        var valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "Cost".equals( serv ) )
      {
        operation = "FIN_Cost_QueryList";
        service = "APINT_FIN_CostsAssociation_WS";
        valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "IDs".equals( serv ) )
      {
        operation = "HelpDesk_IDs_QueryList";
        service = "APINT_HPD_HelpDesk_WS";
        valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "Relationships".equals( serv ) )
      {
        operation = "MG_Relation_QueryList";
        service = "APCMI_CHG_Associations_WS";
        valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "Tasks".equals( serv ) )
      {
        operation = "OpGetList";
        service = "APCMI_TMS_AssocSummaryDataJoin_WS";
        valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
      else if ( "Approvers".equals( serv ) )
      {
        operation = "OpGetList";
        service = "APCMI_CHG_ChangeAPDetailSignature_WS";
        valuesNode = "getListValues";
        return cmo.queryScriptObjectList( operation, service, valuesNode, qual, null, null );
      }
    }
    catch (ex)
    {
      var msg = ex.toString();
      if ( msg.indexOf( "Qualification line error" ) != -1 )
      {
        return null;
      }
      return msg;
    }
  }
}

/**
 * Retrieves the Product Categorization Node
 * Parameters: a java.util.Map<String, String>
 *
 * Return: Product Categorization Node, Node
 */
function queryNode( params )
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var fields = params.get( "fields" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || fields == null )
  {
    return "One of the required fields (cmUser, " +
    "operation, webService, requestNS, responseNS, fields) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    return cmo.queryNode( operation, webService, requestNS, responseNS, fields );
  }
}

/**
 * Retrieves the work logs for a specific incident number (change request ID)
 * Parameters: a java.util.Map<String, String>
 *
 * Return: List<GenericWSObject>
 */
function queryWorkLogs( params )
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var incidentNumber = params.get( "IncidentNumber" );
  if ( user == null  || incidentNumber == null )
  {
    return "One of the required fields (cmUser, IncidentNumber) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    try
    {
      cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
      return cmo.queryWorkLogs( incidentNumber );
    }
    catch (ex)
    {
      ex.toString();
    }
  }
}

/**
 * Sends a Generic Request
 * Parameters: a java.util.Map<String, String>
 *
 * Return: a error message or null if successful
 */
function sendGenericRequest( params )
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var fields = params.get( "fields" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || fields == null )
  {
    return "One of the required fields (cmUser, " +
    "operation, webService, requestNS, responseNS, fields) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = cmo.sendGenericRequest( operation, webService, requestNS, responseNS, fields );
    return escapeMessage( msg );
  }
}

// Sends request where XML contains more than one tier
//  complexMap is of type Map<String, Object> where object should be of type
//  String or Map<String, Object>
function sendComplexRequest( params )
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var operation = params.get( "operation" );
  var webService = params.get( "webService" );
  var requestNS = params.get( "requestNS" );
  var responseNS = params.get( "responseNS" );
  var complexMap = params.get( "complexMap" );
  
  if ( user == null  ||
       operation == null || webService == null || 
       requestNS == null || responseNS == null || complexMap == null )
  {
    return "One of the required fields (cmUser, " +
    "operation, webService, requestNS, responseNS, complexMap) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var msg = cmo.sendComplexRequest( operation, webService, requestNS, responseNS, complexMap );
    return escapeMessage( msg );
  }
}

/**
 * Updates the incident (Change Request)
 * Parameters: a java.util.Map<String, String>, includes WSUpdateFields
 *
 * Return: a error message or null if successful
 */
function updateIncident( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var incident = params.get( "incident" );
  if ( user == null  || incident == null )
  {
    return "One of the required fields (cmUser, Incident) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );
    var operation = "HelpDesk_Modify_Service1";
    var webService = "APINT_HPD_IncidentInterface_WS";
    var msg = cmo.sendGenericRequest( operation, webService, webService, webService, incident.getFields() );
    return escapeMessage( msg );
  }
}

/**
 * Used to validate a user SD credentials
 *
 * Returns: null if the validation was successful and a String with the error message otherwise
 */
function validateCreds(params)
{
  try
  {
    //ChangeManagementObject( String url, String serverName, String user,
    //        String password, String authentication, String locale,
    //        String timeZone )
    var user = params.get( "cmUser" );
    var password = params.get( "cmPass" );    
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, password, null, null, null );
    return cmo.validateCreds( user, password );
  }
  catch(e)
  {
    return e.toString();
  }
  return null; 
}

/**
 * Updates the Approver Status
 * Parameters: Request_ID and Approver status
 *
 * Returns: null if the validation was successful and a String with the error message otherwise
 */
function updateApproverStatus( params ) 
{
  var user = params.get( "cmUser" );
  var pass = params.get( "cmPass" );
  var requestID = params.get( "Request_ID" );
  var approvalStatus = params.get( "Approval_Status2" );
  if ( user == null  || requestID == null || approvalStatus == null )
  {
    return "One of the required fields (cmUser, Request_ID, Approval_Status2) has no value.";
  }
  else
  {
    var cmo = new ChangeManagementObject();
    cmo.initClient( remedyURL, remedyServerName, user, pass, null, null, null );

    var msg = cmo.updateApproverStatus( requestID, approvalStatus);
    return escapeMessage( msg );
  }
}
